var stats = (function() {

	var defaultStopWords = "a about above across after again against all almost alone along already also although always among an and another any anybody anyone anything anywhere are area areas around as ask asked asking asks at away b back backed backing backs be became because become becomes been before began behind being beings best better between big both but by c came can cannot case cases certain certainly clear clearly come could d did differ different differently do does done down down downed downing downs during e each early either end ended ending ends enough even evenly ever every everybody everyone everything everywhere f face faces fact facts far felt few find finds first for four from full fully further furthered furthering furthers g gave general generally get gets give given gives go going good goods got great greater greatest group grouped grouping groups h had has have having he her here herself high high high higher highest him himself his how however i if important in interest interested interesting interests into is it its itself j just k keep keeps kind knew know known knows l large largely last later latest least less let lets like likely long longer longest m made make making man many may me member members men might more most mostly mr mrs much must my myself n necessary need needed needing needs never new new newer newest next no nobody non noone not nothing now nowhere number numbers o of off often old older oldest on once one only open opened opening opens or order ordered ordering orders other others our out over p part parted parting parts per perhaps place places point pointed pointing points possible present presented presenting presents problem problems put puts q quite r rather really right right room rooms s said same saw say says second seconds see seem seemed seeming seems sees several shall she should show showed showing shows side sides since small smaller smallest so some somebody someone something somewhere state states still still such sure t take taken than that the their them then there therefore these they thing things think thinks this those though thought thoughts three through thus to today together too took toward turn turned turning turns two u under until up upon us use used uses v very w want wanted wanting wants was way ways we well wells went were what when where whether which while who whole whose why will with within without work worked working works would x y year years yet you young younger youngest your yours z";
	
	// get options from settings if available
	var pageCountOptions =  typeof _settings != "undefined" && _settings.standard_pages ? _settings.standard_pages : {
		lines_per_page: 30,
		chars_per_line: 60,
		remove_blanks: true
	};
	var readingTimeWPM = typeof _settings != "undefined" && _settings.reading_wpm ? _settings.reading_wpm : 190;
	var speakingTimeWPM = typeof _settings != "undefined" && _settings.speaking_wpm ? _settings.speaking_wpm : 120;
	

	var cleanedText = "";

	// put stats logic in sep module and use importScripts('foo.js') here 
	// stats can be used elswere, not just in infobar
	function getCharactersWS(text) {
		text = text ? cleanText(text) : cleanedText;
		return {
			label: "C",
			unit: "characters w/ spaces",
			count: text.length
		};
	}

	function getCharactersWOS(text) {
		text = text ? cleanText(text) : cleanedText;
		return {
			label: "C",
			unit: "characters w/o spaces",
			count: text.replace(/\s/g, "").length
		};
	}

	function getWords(text) {
		text = text ? cleanText(text) : cleanedText;
		var m = text.match(/\S+/g);
		return {
			label: "W",
			unit: "words",
			count: (m instanceof Array) ? m.length : 0,
			percent: 0
		};
	}

	function getParagraphs(text) {
		text = text ? cleanText(text) : cleanedText;
		var m = (text + "\n").match(/.+\n/g);
		// return (m instanceof Array) ? m.length : 0;

		return {
			label: "PR",
			unit: "paragraphs",
			count: (m instanceof Array) ? m.length : 0,
			percent: 0
		}
	}

	
	function getSentences(text) {
		text = text ? cleanText(text) : cleanedText;
		return {
			label: "S",
			unit: "sentences",
			count: text.replace(/[^\.!?]/g, '').length || 1,
			percent: 0
		}		
	}

	
	function getPages(text) {
		text = text ? cleanText(text) : cleanedText;

		var pages = 0;
		var lines = 0;
		var linesOnPage = 0;
		var currentPage = 0;

		var linesA = wordwrap(text, pageCountOptions.chars_per_line, "\n", true, pageCountOptions.remove_blanks);

		var max = linesA.length;
		for (var i = 0; i < max; i++) {
			lines++;
			linesOnPage++;
			if (linesOnPage >= pageCountOptions.lines_per_page) {
				pages++;
				linesOnPage = 0;
			} else if (/^[\+\*]{3,}/.test(linesA[i])) {
				pages++;
				linesOnPage = 0;
				lines--;
			}
		}

		var lastPagePercent = linesOnPage / pageCountOptions.lines_per_page;


		return {
			unit: "standard pages",
			label: "PG",
			count: (pages + lastPagePercent).toFixed(1),
			lines: lines,
			currentPage: currentPage
		};

	}

	function getReadingTime(text) {
		text = text ? cleanText(text) : cleanedText;		
		return {
			unit: "reading time",
			label: "RD",
			count: seconds2time((getWords(text).count / readingTimeWPM * 60).toFixed(0))
		};
	}

	function getSpeakingTime(text) {
		text = text ? cleanText(text) : cleanedText;
		return {
			unit: "speaking time",
			label: "SPK",
			count: seconds2time((getWords(text).count / speakingTimeWPM * 60).toFixed(0))
		};
	}


	function getKeyWords(text, options, kA) {

		var defaultOptions = {
			wordsReturned: 25,
			frequencyWeight: 100,
			positionWeight: 10,
			complexWeight: 30,
			stopWords: defaultStopWords
		};

		if (typeof options == 'object') {
			options = $.extend(defaultOptions, options);
		} else {
			options = defaultOptions;
		}
		text = text ? cleanText(text) : cleanedText;


		// get keywords if not provided
		kA = kA ? combineSimilarWords(kA, { stopWords: options.stopWords }) : combineSimilarWords(getUniqueWords(text, { stopWords: options.stopWords }));
		

		for (var i = 0; i < kA.length; i++) {
			var index = 0;
			// freq
			index += mapNumberToRange(kA[i].count, [1, kA[0].count + 1], [1, options.frequencyWeight]);
			// position
			index += mapNumberToRange(kA[i].positions[1], [1, kA[i].positions[0]], [options.positionWeight, 0]);
			// complex
			index += mapNumberToRange(kA[i].syllables, [1, 10], [1, options.complexWeight]);

			kA[i].index = index;
		}

		kA.sort(function(o1, o2) {
			return o2.index - o1.index;
		});

		return kA.map(function(item) {
			return {
				label: item.label,
				index: item.index
			}
		}).slice(0, options.wordsReturned);

	}


	function getUniqueWords(text, options) {


		var defaultOptions = {
			minWordLength: 3,
			minFreq: 1,
			maxHits: 100,
			combineSimilar: true, // fake stemming
			sortAlpha: false,
			excludeStopWords: true,
			stopWords: defaultStopWords
		};

		if (typeof options == 'object') {
			options = $.extend(defaultOptions, options);
		} else {
			options = defaultOptions;
		}

		text = text ? cleanText(text) : cleanedText;

		var wordTokens = text.split(/\s+/);

		
		var distinctA = [];

		for (var i = 0; i < wordTokens.length; i++) {
			var word = wordTokens[i].toLowerCase().trim();

			// remove punctuations
			word = word.replace(/[~`!@#$%^&*(){}\[\];:"'“”«»<,.>?\/\\|_+=]/g, "")	
			.replace(/^[-'‘]+/g, "") // remove if at the front of the word
			.replace(/[-'’]s*$/g, ""); // remove if at the end of the word



			// remove numbers
			if (!isNaN(word)) {
				continue;
			}
			
			// remove short words <2c
			if (word.length < options.minWordLength) {
				continue;
			}

			//remove stop words
			var isStopWord = options.stopWords.split(" ").indexOf(word.toLowerCase());
			if (options.excludeStopWords && isStopWord >= 0) {
				continue;
			}

			// check if already in unique array
			var wordAlreadyInArray = false;
			
			for (var ii = 0; ii < distinctA.length; ii++) {
				if (distinctA[ii].label == word) {
					distinctA[ii].count = distinctA[ii].count + 1;
					distinctA[ii].positions.push(i);
					wordAlreadyInArray = true;
					break;
				}
			}

			// if not in array, add new
			if (!wordAlreadyInArray) {
				distinctA.push({label: word, count: 1, syllables: countSyllables(word), stopWord: isStopWord >= 0 ? true : false,positions: [wordTokens.length, i] });
			}


		}

		// remove min freq
		distinctA = distinctA.filter( function(word) { return word.count >= options.minFreq });

		// sort frequent first
		distinctA.sort(function(o1, o2) {
			return o2.count - o1.count;
		});

		// max hits
		distinctA = distinctA.slice(0, options.maxHits);

		// sort alphabetically
		if (options.sortAlpha) {
			distinctA.sort(function(o1, o2) {
				if (o1.label < o2.label) return -1;
				if (o1.label > o2.label) return 1;
				return 0;
			});
		}
		return distinctA;

	}

	function combineSimilarWords(fA) {

		// alphabetical
		fA = fA.sort(function(o1, o2) {
		    if (o1.label < o2.label) return -1;
		    if (o1.label > o2.label) return 1;
		    return 0;
		});


		for (var i = fA.length - 1; i >= 1; i--) {
			if (stemmer(fA[i].label) == stemmer(fA[i - 1].label)) {
				if (fA[i].label.length > fA[i - 1].label.length) {
					fA[i - 1].count += fA[i].count;
					fA[i - 1].positions = fA[i - 1].positions.concat(fA[i].positions).sort(function(a, b) {return b - a});
					fA.splice(i, 1);
				} else if (fA[i].label.length < fA[i - 1].label.length) {
					fA[i].count += fA[i - 1].count;
					fA[i].positions = fA[i].positions.concat(fA[i - 1].positions).sort(function(a, b) {return b - a});
					fA.splice(i - 1, 1);
				}
			}
		}

		// and resort by count
		fA.sort(function(o1, o2) {
			return o2.count - o1.count;
		});

		return fA;
	}


	// function getUniqueWords(text) {}

	// function getHardWords(text) {}

	// function getAvgWordsPerSentence(text) {}

	// function getFleschKincaidReadingEase(text) {}

	// function getFleschKincaidGradeLevel(text) {}

	// function getGunningFogScore(text) {}

	// function getColemanLiauIndex(text) {}

	// function getSmogIndex(text) {}

	// function getAutomatedReadabilityIndex(text) {}




	/*
	##     ##  ########   ####  ##         
	##     ##     ##       ##   ##         
	##     ##     ##       ##   ##         
	##     ##     ##       ##   ##         
	##     ##     ##       ##   ##         
	##     ##     ##       ##   ##         
	 #######      ##      ####  ########   
	*/
	function cleanText(text) {

		text = text
			.replace(/[^:]\/{2}.*?\/{2}|([^:]\/{2}.+$)/gm, "") // Strip comments
			.replace(/~{2}.+~{2}/g, "") // Strip strike out text
			.replace(/^#{1,6}/gm, ""); // Replace special
			//.replace(/(?:https?|ftp):\/\/[\n\S]+/g, ''); // Replace URLs
			// .replace(/<[^>]+>/g, "")
			// .replace(/[,:;()\-]/, " ") // Replace commans, hyphens etc (count them as spaces)
			// .replace(/[\.!?]/, ".") // Unify terminators
			// .replace(/^\s+/, "") // Strip leading whitespace
			// .replace(/[ ]*(\n|\r\n|\r)[ ]*/, " ") // Replace new lines with spaces
			// .replace(/([\.])[\. ]+/, ".") // Check for duplicated terminators
			// .replace(/[ ]*([\.])/, ". ") // Pad sentence terminators
			// .replace(/\s+/, " ") // Remove multiple spaces
			// .replace(/\s+$/, ""); // Strip trailing whitespace

		return removeMetaPart(text);
	}




	// remove meta data portion of the string
	function removeMetaPart(text) {

		var isMeta = true;
		
		while (isMeta) {
			if (text.match(/^\w+:.*\n/)) {
				text = text.substring(text.indexOf("\n") + 1); // remove this line
			} else {
				isMeta = false;
			}
		}
		return text.trim();
	}



	// add tousands separator
	function add000separator(x) {
		return x.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
	}

	// wordwrap at column
	function wordwrap(str, int_width, str_break, cut, remove_blanks) {
		var m = ((arguments.length >= 2) ? arguments[1] : 60);
		var b = ((arguments.length >= 3) ? arguments[2] : "\n");
		var c = ((arguments.length >= 4) ? arguments[3] : false);

		var i, j, l, s, r;

		str += '';

		if (m < 1) {
			return str;
		}

		for (i = -1, l = (r = str.split(/\r\n|\n|\r/)).length; ++i < l; r[i] += s) {
			for (s = r[i], r[i] = ""; s.length > m; r[i] += s.slice(0, j) + ((s = s.slice(j)).length ? b : "")) {
				j = c == 2 || (j = s.slice(0, m + 1).match(/\S*(\s)?$/))[1] ? m : j.input.length - j[0].length || c == 1 && m || j.input.length + (j = s.slice(m).match(/^\S*/)).input.length;
			}
		}

		var trimmed = (remove_blanks) ? trimDoubles(r.join("\n")) : r.join("\n");
		return trimmed.split(/\r\n|\r|\n/);
	}

	// trim multiple new line chars (\n)
	function trimDoubles(str) {
		return str.replace(/\n{2,}/mg, "\n");
	}


	// convert seconds to time string
	function seconds2time(seconds) {
		var hours = Math.floor(seconds / 3600);
		var minutes = Math.floor((seconds - (hours * 3600)) / 60);
		var seconds = seconds - (hours * 3600) - (minutes * 60);
		var time = "";

		if (hours != 0) {
			time = hours + ":";
		}
		if (minutes != 0 || time !== "") {
			minutes = (minutes < 10 && time !== "") ? "0" + minutes : String(minutes);
			time += minutes + ":";
		}
		if (time === "") {
			time = "0:" + seconds;
		} else {
			time += (seconds < 10) ? "0" + seconds : String(seconds);
		}
		return time;
	}



	function countSyllables(word) {
		var syllableCount = 0,
			prefixSuffixCount = 0,
			wordPartCount = 0;

		// Prepare word - make lower case and remove non-word characters
		word = word.toLowerCase().replace(/[^a-z]/g, "");

		// Specific common exceptions that don't follow the rule set below are handled individually
		// Array of problem words (with word as key, syllable count as value)
		var problemWords = {
			"simile": 3,
			"forever": 3,
			"shoreline": 2
		};

		// Return if we've hit one of those...
		if (problemWords.hasOwnProperty(word)) return problemWords[word];

		// These syllables would be counted as two but should be one
		var subSyllables = [
			/cial/,
			/tia/,
			/cius/,
			/cious/,
			/giu/,
			/ion/,
			/iou/,
			/sia$/,
			/[^aeiuoyt]{2,}ed$/,
			/.ely$/,
			/[cg]h?e[rsd]?$/,
			/rved?$/,
			/[aeiouy][dt]es?$/,
			/[aeiouy][^aeiouydt]e[rsd]?$/,
			/^[dr]e[aeiou][^aeiou]+$/, // Sorts out deal, deign etc
			/[aeiouy]rse$/ // Purse, hearse
		];

		// These syllables would be counted as one but should be two
		var addSyllables = [
			/ia/,
			/riet/,
			/dien/,
			/iu/,
			/io/,
			/ii/,
			/[aeiouym]bl$/,
			/[aeiou]{3}/,
			/^mc/,
			/ism$/,
			/([^aeiouy])\1l$/,
			/[^l]lien/,
			/^coa[dglx]./,
			/[^gq]ua[^auieo]/,
			/dnt$/,
			/uity$/,
			/ie(r|st)$/
		];

		// Single syllable prefixes and suffixes
		var prefixSuffix = [
			/^un/,
			/^fore/,
			/ly$/,
			/less$/,
			/ful$/,
			/ers?$/,
			/ings?$/
		];

		// Remove prefixes and suffixes and count how many were taken
		prefixSuffix.forEach(function(regex) {
			if (word.match(regex)) {
				word = word.replace(regex, "");
				prefixSuffixCount++;
			}
		});

		wordPartCount = word
			.split(/[^aeiouy]+/ig)
			.filter(function(wordPart) {
				return !!wordPart.replace(/\s+/ig, "").length;
			})
			.length;

		// Get preliminary syllable count...
		syllableCount = wordPartCount + prefixSuffixCount;

		// Some syllables do not follow normal rules - check for them
		subSyllables.forEach(function(syllable) {
			if (word.match(syllable)) syllableCount--;
		});

		addSyllables.forEach(function(syllable) {
			if (word.match(syllable)) syllableCount++;
		});

		return syllableCount || 1;
	}


var stemmer=function(){var l={ational:"ate",tional:"tion",enci:"ence",anci:"ance",izer:"ize",bli:"ble",alli:"al",entli:"ent",eli:"e",ousli:"ous",ization:"ize",ation:"ate",ator:"ate",alism:"al",iveness:"ive",fulness:"ful",ousness:"ous",aliti:"al",iviti:"ive",biliti:"ble",logi:"log"},m={icate:"ic",ative:"",alize:"al",iciti:"ic",ical:"ic",ful:"",ness:""},g=/^([^aeiou][^aeiouy]*)?[aeiouy][aeiou]*[^aeiou][^aeiouy]*/,e=/^([^aeiou][^aeiouy]*)?[aeiouy][aeiou]*[^aeiou][^aeiouy]*[aeiouy][aeiou]*[^aeiou][^aeiouy]*/,
n=/^([^aeiou][^aeiouy]*)?[aeiouy][aeiou]*[^aeiou][^aeiouy]*([aeiouy][aeiou]*)?$/,p=/^([^aeiou][^aeiouy]*)?[aeiouy]/,q=/^(.+?)(ss|i)es$/,r=/^(.+?)([^s])s$/,t=/^(.+?)eed$/,u=/^(.+?)(ed|ing)$/,h=/.$/,v=/(at|bl|iz)$/,w=/([^aeiouylsz])\1$/,x=/^[^aeiou][^aeiouy]*[aeiouy][^aeiouwxy]$/,y=/^(.+?[^aeiou])y$/,z=/^(.+?)(ational|tional|enci|anci|izer|bli|alli|entli|eli|ousli|ization|ation|ator|alism|iveness|fulness|ousness|aliti|iviti|biliti|logi)$/,A=/^(.+?)(icate|ative|alize|iciti|ical|ful|ness)$/,B=/^(.+?)(al|ance|ence|er|ic|able|ible|ant|ement|ment|ent|ou|ism|ate|iti|ous|ive|ize)$/,
C=/^(.+?)(s|t)(ion)$/,D=/^(.+?)e$/,E=/ll$/,F=/^[^aeiou][^aeiouy]*[aeiouy][^aeiouwxy]$/;return function(a){if(3>a.length)return a;var f=a.substr(0,1);"y"==f&&(a=f.toUpperCase()+a.substr(1));var c=q;var b=r;c.test(a)?a=a.replace(c,"$1$2"):b.test(a)&&(a=a.replace(b,"$1$2"));c=t;b=u;if(c.test(a))b=c.exec(a),c=g,c.test(b[1])&&(c=h,a=a.replace(c,""));else if(b.test(a)){b=b.exec(a);var d=b[1];b=p;if(b.test(d)){a=d;b=v;var k=w;b.test(a)?a+="e":k.test(a)?(c=h,a=a.replace(c,"")):x.test(a)&&(a+="e")}}c=y;c.test(a)&&
(b=c.exec(a),d=b[1],a=d+"i");c=z;c.test(a)&&(b=c.exec(a),d=b[1],b=b[2],c=g,c.test(d)&&(a=d+l[b]));c=A;c.test(a)&&(b=c.exec(a),d=b[1],b=b[2],c=g,c.test(d)&&(a=d+m[b]));c=B;b=C;c.test(a)?(b=c.exec(a),d=b[1],c=e,c.test(d)&&(a=d)):b.test(a)&&(b=b.exec(a),d=b[1]+b[2],b=e,b.test(d)&&(a=d));c=D;c.test(a)&&(b=c.exec(a),d=b[1],c=e,b=n,k=F,c.test(d)||b.test(d)&&!k.test(d))&&(a=d);c=E;b=e;c.test(a)&&b.test(a)&&(c=h,a=a.replace(c,""));"y"==f&&(a=f.toLowerCase()+a.substr(1));return a}}();

	


	/*
	   ###     ########   ####  
	  ## ##    ##     ##   ##   
	 ##   ##   ##     ##   ##   
	##     ##  ########    ##   
	#########  ##          ##   
	##     ##  ##          ##   
	##     ##  ##         ####  
	*/
	return {
		setReadingTime: (function(rt) {readingTimeWPM = rt}),
		setSpeakingTime: (function(st) {speakingTimeWPM = st}),
		setStandardPages: (function(sp) {pageCountOptions = sp}),
		getCharactersWS: getCharactersWS,
		getCharactersWOS: getCharactersWOS,
		getWords: getWords,
		getParagraphs: getParagraphs,
		getSentences: getSentences,
		getPages: getPages,
		getReadingTime: getReadingTime,
		getSpeakingTime: getSpeakingTime,
		getUniqueWords: getUniqueWords,
		combineSimilarWords: combineSimilarWords,
		getKeyWords: getKeyWords,
		add000separator: add000separator, // this one should not be exposed
		removeMetaPart: removeMetaPart
	};

})();