module.exports = function(api, pluginInfo) {

	var options = pluginInfo.options;
	var $ = api.window.$;
	var m = window;

	var updateSessionPomodorosTimer = null;
	var sessionPomodoros = [];
	var current_pomodoro_sequence = 0;

	function _init() {
		updateSessionPomodorosTimer = m.setInterval(updateSessionPomodoros, 1000);
	}



	function getNewPomodoro() {
		
		return {
			type: options.pomodoro_sequence[current_pomodoro_sequence].type,
			duration: options.pomodoro_sequence[current_pomodoro_sequence].duration,
			status: "running", // completed, void
			document_id: m._currentDoc._id.toString(),
			project_id: m._currentDoc.pid,
			session_id: m._currentDoc.ses.startTime, // session start time
			start_time: new Date().getTime(),
			end_time: -1,
			initial_count: m.stats.getWords(m._editor.getValue()).count,
			end_count: 0
		};
	}


	function startNewPomodoro() {
		// set next sequence member
		current_pomodoro_sequence = sessionPomodoros.length % options.pomodoro_sequence.length;
		var newPomodoro = getNewPomodoro();
		sessionPomodoros.push(newPomodoro);
		m._currentDoc.ses.pomodoroSess = sessionPomodoros; // also add session data to the doc
	}




	function endCurrentPomodoro() {
		sessionPomodoros[sessionPomodoros.length - 1].status = "completed";
		sessionPomodoros[sessionPomodoros.length - 1].end_time = new Date().getTime();
		sessionPomodoros[sessionPomodoros.length - 1].end_count = m.stats.getWords(m._editor.getValue()).count;
	}
	



	function updateSessionPomodoros() {

		if (sessionPomodoros.length < 1) return;

		var pStart = sessionPomodoros[sessionPomodoros.length - 1].start_time;
		var pDuration = sessionPomodoros[sessionPomodoros.length - 1].duration;
		var pInitialCount = sessionPomodoros[sessionPomodoros.length - 1].initial_count;

		
		var now = m.moment(new Date().getTime()).subtract(pDuration, 'minutes');
		
		//var hr = m.moment(now).diff(pStart, "hours");
		var min = m.moment(now).diff(pStart, "minutes");
		var sec = m.moment(now).diff(pStart, "seconds") - (min * 60);

	
		var minLeft = min > 0 ? 0 : Math.abs(min);
		var secLeft = sec > 0 ? 0 : Math.abs(sec);
		
		if (minLeft == 0 && secLeft == 0) {
			playSound();
			endCurrentPomodoro();
			startNewPomodoro();
			
			$('#pomodoro_infobar_container').remove();
			$('#info_bar_top_left').prepend(getInfobarHtml());
		}
		
		var secLeftString = secLeft < 10 ? '0' + secLeft : secLeft;
		secLeftString = minLeft < 1 ? ':' + secLeftString : "";

		var count = m._currentWordCount - pInitialCount; // < 0 ? 0 : m._currentWordCount - pInitialCount;
		$('#pomodoro_infobar_time_label').text(minLeft + secLeftString);
		$('#pomodoro_infobar_count_label').html(m.stats.add000separator(count) + '&nbsp;&nbsp;&#8226;&nbsp;&nbsp;');
	}


	function getInfobarHtml() {
		
		var container = $('<span/>', { id: 'pomodoro_infobar_container' });
		var timeLabel = $('<span/>', { id: 'pomodoro_infobar_time_label' });
		var completedLabel = $('<span/>', { id: 'pomodoro_infobar_completed' });
		var countLabel = $('<span/>', { id: 'pomodoro_infobar_count_label' });

		var completed = 0;




		var workPomodoros = sessionPomodoros.filter(function(item) { return item.type == "work" && item.status == "completed"; });

		var min = Math.min.apply(null, workPomodoros.map(function(item) { return item.end_count - item.initial_count; }));
		var max = Math.max.apply(null, workPomodoros.map(function(item) { return item.end_count - item.initial_count; }));
		min = min <= 0 ? 0 : min;

		var normalMax = 1.2;
		var normalMin = .2;


		for (var i = 0; i < sessionPomodoros.length; i++) {
			
			var pomodoroIcon = options.full_pomodoro_icon; // default full pomodoro

			if (i == sessionPomodoros.length - 1) { // last
				if (sessionPomodoros[i].type == "work") {
					pomodoroIcon = options.write_pomodoro_icon;
				} else if (sessionPomodoros[i].type == "break") {
					pomodoroIcon = options.break_pomodoro_icon;
				}else if (sessionPomodoros[i].type == "long_break") {
					pomodoroIcon = options.longbreak_pomodoro_icon;
				} 
			}
			// only show pomodoro if it is work, ignore breaks
			if (sessionPomodoros[i].type == "work" || i == sessionPomodoros.length - 1) {

				var words = sessionPomodoros[i].end_count - sessionPomodoros[i].initial_count;
				words = words < 0 ? 0 : words;
				var scaledValue = ((normalMax - normalMin) / (max - min) * (words - max) + normalMax).toFixed(3);

				scaledValue = i == sessionPomodoros.length - 1 ? 1 : scaledValue;


				var pomodoro = $('<span>' + pomodoroIcon + '</span>').addClass('pomodoro_infobar theme-mark-color').css('font-size', (scaledValue * 100) + '%');
				
				pomodoro.jqxTooltip({ theme: 'metro', autoHide: true, showArrow: false, animationShowDelay: 100, animationHideDelay: 100, autoHideDelay: 3000, opacity: 1, content: words.toString() + " words", position: 'mouse', name: 'pomo', trigger: 'hover', closeOnClick: true });
				
				container.append(pomodoro);
				completed++;
			}
		}
		completedLabel.text((completed - 1) + " ");
		return container.prepend(completedLabel).append(countLabel).append(timeLabel);
	}



	function playSound() {
		if (options.play_sounds) {
			var s = new m.Howl({ urls: [pluginInfo.plugin_dir_path + "/beepbeep.wav"] });
			s.play();
		}
	}



	// Start new pomodoro session
	m.document.body.addEventListener("newSessionE", function(e) {
		$('#pomodoro_infobar_container').remove();
		sessionPomodoros = [];
		startNewPomodoro();		

		$('#info_bar_top_left').prepend(getInfobarHtml());
		playSound();
	}, false);


	// Stop new pomodoro session
	m.document.body.addEventListener("stopSessionE", function(e) {
		sessionPomodoros = [];
		$('#pomodoro_infobar_container').remove();
	}, false);



	// Update on doc swap
	m._editor.on("swapDoc", function() {

		$("#info_bar_top_left").hide().delay(1700).show("slow");

		if (m._currentDoc.ses && m._currentDoc.ses.pomodoroSess && m._currentDoc.ses.pomodoroSess.length >= 1) {
			sessionPomodoros = JSON.parse(JSON.stringify(m._currentDoc.ses.pomodoroSess));
			$('#pomodoro_infobar_container').remove();
			$('#info_bar_top_left').prepend(getInfobarHtml());

		} else {
			sessionPomodoros = [];
			$('#pomodoro_infobar_container').remove();
		}
		// updateSessionPomodoros();

	});




	return {
		_init: _init
	};
};