module.exports = function(api, pluginInfo) {

	var monkey = window;
	var options = pluginInfo.options;
	var $ = api.window.$;


	var bcHex = "#ffffff";
	var fcHex = "#000000";

	var $iframe;
	var paneHandle;
	var templateCSS = "";
	var wmColorsCSS = "";
	var templateJS = "";

	var numberedHeadingsCSS = "<style type=\"text/css\"> body{counter-reset: section;} h2{counter-reset: sub-section;} h3{counter-reset: composite;} h4{counter-reset: detail;} h5{counter-reset: sub-detail;} h2:before{ counter-increment: section; content: counter(section) \". \"; } h3:before{ counter-increment: sub-section; content: counter(section) \".\" counter(sub-section) \". \"; } h4:before{ counter-increment: composite; content: counter(section) \".\" counter(sub-section) \".\" counter(composite) \". \"; } h5:before{ counter-increment: detail; content: counter(section) \".\" counter(sub-section) \".\" counter(composite) \".\" counter(detail) \". \"; } h6:before{ counter-increment: sub-detail; content: counter(section) \".\" counter(sub-section) \".\" counter(composite) \".\" counter(detail) \".\" counter(sub-detail) \". \"; }</style>";

	// var elasticFontSizeCSS = '<style>html, table, code {font-size: 4vw!important}</style>';

	var renderer = new monkey.marked.Renderer();
	monkey.marked.setOptions({
		renderer: renderer,
		gfm: true,
		tables: true,
		breaks: true,
		pedantic: false,
		sanitize: false,
		smartLists: true,
		smartypants: true
	});

	

	function _init() {
		setPane();
		createIframe();
		setPaletteItems();
	}

	function setPane() {
		paneHandle = monkey.universalPane.registerPane("wm_preview", "Markdown preview", {
			backgroundColor: "#ffffff"
			// width: 400
		});
	}

	function getContextMenu() {
		
		var previewContextMenu = new monkey.gui.Menu({
			'type': 'menubar'
		});

		previewContextMenu.append(new monkey.gui.MenuItem({
			label: 'Select template',
			submenu: getTemplatesMenu()
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			label: 'Print',
			click: function() {
				print();
			}
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			label: 'Open in browser',
			click: function() {
				saveTempAndExport("html");
			}
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			label: 'Open in .doc editor',
			click: function() {
				saveTempAndExport("doc");
			}
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			label: 'Update',
			click: function() {
				updatePreview();
			}
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			type: 'separator',
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			type: 'checkbox',
			checked: monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] == "1" ? true : false,
			label: 'Include comments',
			click: function() {
				if (monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] && monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] == "1") {
					monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] = "0";
				} else {
					monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] = "1";
				}
				updatePreview();
			}
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			type: 'checkbox',
			checked: monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] == "1" ? true : false,
			label: 'Numbered headings',
			click: function() {
				if (monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] && monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] == "1") {
					monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] = "0";
				} else {
					monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] = "1";
				}
				updatePreview();
			}
			
		}));

		previewContextMenu.append(new monkey.gui.MenuItem({
			type: 'checkbox',
			checked: monkey.localStorage["EXPORT_SYNC_SCROLLING"] == "1" ? true : false,
			label: 'Synchronize scrolling',
			click: function() {
				if (monkey.localStorage["EXPORT_SYNC_SCROLLING"] && monkey.localStorage["EXPORT_SYNC_SCROLLING"] == "1") {
					monkey.localStorage["EXPORT_SYNC_SCROLLING"] = "0";
				} else {
					monkey.localStorage["EXPORT_SYNC_SCROLLING"] = "1";
				}
				updatePreview();
			}
		}));


		// previewContextMenu.append(new monkey.gui.MenuItem({
		// 	type: 'checkbox',
		// 	checked: monkey.localStorage["EXPORT_ELASTIC_FONT_SIZE"] == "1" ? true : false,
		// 	label: 'Elastic font size',
		// 	click: function() {
		// 		if (monkey.localStorage["EXPORT_ELASTIC_FONT_SIZE"] && monkey.localStorage["EXPORT_ELASTIC_FONT_SIZE"] == "1") {
		// 			monkey.localStorage["EXPORT_ELASTIC_FONT_SIZE"] = "0";
		// 		} else {
		// 			monkey.localStorage["EXPORT_ELASTIC_FONT_SIZE"] = "1";
		// 		}
		// 		updatePreview();
		// 	}
			
		// }));

		return previewContextMenu;
	}


	function getTemplatesMenu() {

		var templatesSubmenu = new monkey.gui.Menu({
			'type': 'menubar'
		});


		var templatesA = monkey.wm.getInstalledTemplateNames();

		for (var i = 0; i < templatesA.length; i++) {

			// exclude non css files
			if (!templatesA[i].endsWith(".css")) continue;

			var tName = templatesA[i].replace(/\..*$/, "");

			var mItem = new monkey.gui.MenuItem({
				type: 'checkbox',
				checked: monkey.localStorage['EXPORT_TEMPLATE'] == templatesA[i] ? true : false,
				label: tName,
				click: function() {
					changeTemplate(this.label + ".css");
				}
			});

			templatesSubmenu.append(mItem);

		}

		return templatesSubmenu;
	}

	


	function setPaletteItems() {


		// set theme selecton
		monkey.commandPalette.registerPaletteItem({
			id: 'preview-set-template',
			string: 'Markdown Preview: Set Export Template',
			shortcut: "",
			action: '',
			type: "sublist",
			sublist: '_plugins["mkdpreview"].getTemplatePaletteItems()',
			persist: false,
			widget: {
				type: "eval",
				value: 'localStorage["EXPORT_TEMPLATE"]'
			}
		});




		// set print command
		monkey.commandPalette.registerPaletteItem({
				id: 'preview-print',
				string: 'Markdown Preview: Export to Printer',
				shortcut: "",
				action: '_plugins["mkdpreview"].print()',
				type: "eval",
				sublist: [],
				persist: false
			}
		);


		// set print command
		monkey.commandPalette.registerPaletteItem({
				id: 'preview-export-html',
				string: 'Markdown Preview: Export to Default Browser',
				shortcut: "",
				action: '_plugins["mkdpreview"].saveTempAndExport("html")',
				type: "eval",
				sublist: [],
				persist: false
			}
		);


		monkey.commandPalette.registerPaletteItem({
				id: 'preview-export-doc',
				string: 'Markdown Preview: Export to Word (default .doc editor)',
				shortcut: "",
				action: '_plugins["mkdpreview"].saveTempAndExport("doc")',
				type: "eval",
				sublist: [],
				persist: false
			}
		);

		monkey.commandPalette.registerPaletteItem({
				id: 'preview-update',
				string: 'Markdown Preview: Update Preview',
				shortcut: "",
				action: '_plugins["mkdpreview"].updatePreview()',
				type: "eval",
				sublist: [],
				persist: false
			}
		);
	}


	// Get palette items for set template
	function getTemplatePaletteItems() {
		var objectsA = [];
		var templatesA = monkey.wm.getInstalledTemplateNames();

		for (var i = 0; i < templatesA.length; i++) {

			// exclude non css files
			if (!templatesA[i].endsWith(".css")) continue;

			var item = {
				string: "Set Template: " + templatesA[i].replace(/\..*$/, ""),
				shortcut: "",
				action: '_plugins["mkdpreview"].changeTemplate("' + templatesA[i] + '")',
				type: "eval",
				sublist: [],
				persist: true
			};
			objectsA.push(item);
		}
		return objectsA;
	}


	// change preview template 
	function changeTemplate(name) {
		monkey.wm.setTemplate(name);
		templateCSS = monkey.wm.getCssFromTemplateFile(name);
		templateJS = monkey.wm.getJsFromTemplateFile(name);
		setColors();
		updatePreview();
	}



	function createIframe() {

		$("#wm_preview").empty();

		$iframe = $('<iframe scrolling="yes" id="preview-iframe" style="margin-left: 0px; background-color: transparent; position: relative; width: 100%; height: calc(100vh - 30px);" nwfaketop nwdisable></iframe>').appendTo("#wm_preview");


		if (monkey.localStorage["EXPORT_TEMPLATE"]) {
			templateCSS = monkey.wm.getCssFromTemplateFile(monkey.localStorage["EXPORT_TEMPLATE"]);
			templateJS = monkey.wm.getJsFromTemplateFile(monkey.localStorage["EXPORT_TEMPLATE"]);
		}
		
		setColors();

	}



	function setColors() {

	
		bcHex = "#FFFFFF";		
		fcHex = "#090909";

		wmColorsCSS = "<style type=\"text/css\"> @media screen {html {font-size: " + options.preview_zoom_ratio + "vw!important;}} body {padding: 10px!important;} html, body, hr, span, applet, object, iframe, h1, h2, h3, h4, h5, h6, p, a, blockquote, pre, abbr, acronym, address, big, cite, code, del, dfn, ins, kbd, q, s, samp, small, strike, sub, sup, tt, var, b, u, i, center, dl, dt, dd, ol, ul, li, fieldset, form, label, legend, table, caption, tbody, tfoot, thead, tr, th, td, article, aside, canvas, details, embed, figure, figcaption, footer, header, hgroup, menu, nav, output, ruby, section, summary, time, mark, audio, video {background-color:[_bck_color_] !important; color:[_txt_color_] !important; border-color:[_txt_color_] !important; } a {color: [_txt_color_] !important; text-decoration: none !important; border: 0px!important;} pre, code {overflow:hidden !important;} img { max-width: 95% !important;} .inline-comment { font-family: monospace; font-size: .9em; border-left: 4px solid #F2D400!important; background-color: #F2E6AA!important; border-radius: 2px; padding: 2px; margin-left: 3px; margin-left: 3px;} .block-comment { font-family: monospace; font-size: .9em; line-height: 1.1em; background-color: #F2E6AA!important; border-left: 4px solid #F2D400!important; border-radius: 3px; padding: 5px; }</style>";
		wmColorsCSS = wmColorsCSS.replace(/\[_bck_color_\]/g, bcHex).replace(/\[_txt_color_\]/g, fcHex);
	}


	function updatePreview() {

		var iframe = $iframe[0];
		var doc = iframe.document;
		
		
		if (iframe.contentDocument) {
			doc = iframe.contentDocument;
		} else if (iframe.contentWindow) {
			doc = iframe.contentWindow.document;
		}
		doc.open();
		doc.writeln(getHtml(monkey._editor.somethingSelected() ? monkey._editor.getSelection() : monkey._editor.getValue()));
		doc.close();

		

		doc.addEventListener('contextmenu', function(e) {
			e.preventDefault();
			var pos = $("#preview-iframe").position();
			getContextMenu().popup(e.x + pos.left, e.y + pos.top);
		}, true);

	}



	function print() {
		updatePreview(); // so last version is printed even if pane is hidden
		$iframe[0].contentWindow.print();
	}


	function getHtml(text) {

		if (!monkey._editor.somethingSelected()) {
			text = text.substring(monkey.storage.getMetaFields().endPoint, text.length);	
		}

		var content = handleComments(text);
		content = window.marked(content);
		

		var num = monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] && monkey.localStorage["EXPORT_NUMBERED_HEADINGS"] == "1" ? numberedHeadingsCSS: "";

		// font-size:4vw!important;
		return '<html style="">\n<head>\n<title>&nbsp;</title>\n<meta http-equiv="Content-Type" content="text/html; charset=utf-8">\n<script>\n' + templateJS + '\n</script>\n<style type="text/css">\n' + templateCSS + '</style>\n' + wmColorsCSS + num + '\n</head>\n<body style="font-size:.8em;>\n<div id="wrapper">\n' + content + '\n</div>\n</body>\n</html>';
	}



	function saveTempAndExport(ext) {
		updatePreview(); // so last version is printed even if pane is hidden
		var tempPath = monkey.os.tmpdir() + "/wm3export." + ext;
		var html = getHtml(monkey._editor.somethingSelected() ? monkey._editor.getSelection() : monkey._editor.getValue());
		monkey.fs.writeFile(tempPath, html, function(error) {
			monkey.opener(tempPath);
		});

	}

	function handleComments(text) {

		text = text.replace(/\/{2}(.*?)\/{2}/gm, monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] == "1" ? '<span class="inline-comment">$1</span>' : "");

		text = text.replace(/^\/{2}(.*?)$/gm, monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] == "1" ? '<div class="block-comment">$1</div>\n\n' : "");

		text = text.replace(/(?!^)[^:]\/{2}\s*(.*?)$/gm, monkey.localStorage["EXPORT_INCLUDE_COMMENTS"] == "1" ? '<span class="inline-comment">$1</span>' : "");

		return text;
	}



	/*
	########   ##     ##  ########   ##    ##   ########    ######    
	##         ##     ##  ##         ###   ##      ##      ##    ##   
	##         ##     ##  ##         ####  ##      ##      ##         
	######     ##     ##  ######     ## ## ##      ##       ######    
	##          ##   ##   ##         ##  ####      ##            ##   
	##           ## ##    ##         ##   ###      ##      ##    ##   
	########      ###     ########   ##    ##      ##       ######    
	*/



	// on change
	var lastUpdateLines = 0;
	monkey._editor.on("change", function(doc) {

		if (!monkey.universalPane.isPaneVisible("wm_preview")) 
			return;


		var lineCount = monkey._editor.lineCount();
		if (lastUpdateLines != lineCount) {
			setTimeout(function() { updatePreview(); lastUpdateLines = lineCount; }, 1);
		}
	});


	monkey.document.body.addEventListener("switchPaneE", function(e) {
		if (e.detail.newId == "wm_preview") {
			updatePreview();
		} else {
			
		}
	}, false);


	// document load / swap
	monkey._editor.on("swapDoc", function(sevent) {
		if (monkey.universalPane.isPaneVisible("wm_preview"))
			setTimeout(function() { updatePreview(); }, 1);
	});


	// update on save
	monkey.document.body.addEventListener("saveE", function() {
		setTimeout(function() {
			updatePreview();
		}, 1);
	}, false);


	// scroll
	monkey._editor.on("scroll", function(sevent) {

		if (!monkey.universalPane.isPaneVisible("wm_preview") || monkey.localStorage["EXPORT_SYNC_SCROLLING"] != "1")
			return;


		var originPos = monkey._editor.doc.scrollTop;
		var originHeight = monkey._editor.getScrollerElement().scrollHeight; //monkey._editor.doc.height; // monkey._editor.getScrollInfo().height;
		var targetHeight = $iframe[0].contentWindow.document.body.scrollHeight;
		var offset = 0;

		var origPercent = (originPos - offset) / originHeight;
		var targetPos = targetHeight * origPercent;

		$iframe[0].contentWindow.scrollTo(0, targetPos);

	});


	

	
	return {
		_init: _init,
		updatePreview: updatePreview,
		print: print,
		saveTempAndExport: saveTempAndExport,
		getTemplatePaletteItems: getTemplatePaletteItems,
		changeTemplate: changeTemplate
	};

};