module.exports = function(api, pluginInfo) {


	var options = pluginInfo.options;
	var $ = api.window.$;
	var m = window;
	var cm = m._editor;


	var lastPosition = 0;


	function _init() {
		setPaletteCommands();
		if (m.localStorage.getItem("VIEW_TYPEWRITER_SCROLLING_PLUS") === null) {
			m.localStorage["VIEW_TYPEWRITER_SCROLLING_PLUS"] = 0;
		}
	}


	function setPaletteCommands() {
		m.commandPalette.registerPaletteItem({
			id: 'typewriter_scrolling_plus',
			string: 'View: Toggle Typewriter Scrolling Plus on / off',
			shortcut: "",
			action: '_plugins["typewriter_scrolling_plus"].toggleTSP()',
			type: 'eval',
			sublist: [],
			persist: false,
			icon: "&#xf06e;",
			widget: {
				type: "setting",
				value: "VIEW_TYPEWRITER_SCROLLING_PLUS"
			}
		});
	}


	function toggleTSP() {
		if (m.localStorage["VIEW_TYPEWRITER_SCROLLING_PLUS"] == 1) {
			m.localStorage["VIEW_TYPEWRITER_SCROLLING_PLUS"] = 0;
			disableTSP();
			m.wm.showNotice("Typewriter scrolling Plus is disabled", true);

		} else {
			m.localStorage["VIEW_TYPEWRITER_SCROLLING_PLUS"] = 1;
			enableTSP();
			m.wm.showNotice("Typewriter scrolling Plus is enabled", true);

		}
	}

	m.win.on('enter-fullscreen', function() {
		enableTSP();
	});

	m.win.on('leave-fullscreen', function() {
		disableTSP();
	});


	$('#grid').on('click', function(e) {
		if (e.target !== this || !isFullScreen()) return;
		moveTextColumn(0, 300);
	});


	function enableTSP() {
		// scroll to cursor
		scrollTSP(cm, null);
		cm.on("change", scrollTSP);
	}


	function disableTSP() {
		resetUI();
		cm.off("change", scrollTSP);
	}


	function resetUI() {
		moveTextColumn(0, 0);
	}

	function scrollTSP(cm, obj) {

		if (isFullScreen()) {
			var cursorPos = cm.cursorCoords(true, "local");
			var w = $("#editor-placeholder").width();
			var np = w / 2 - cursorPos.left - 20;

			if (Math.abs(np - lastPosition) <= cm.defaultCharWidth() * 5) {
				moveTextColumn(np, 0);
			} else {
				moveTextColumn(np, options.carriage_return_speed);
			}
			lastPosition = np;
		} else {
			resetUI();
		}
	}


	function moveTextColumn(left, speed) {
		if (speed === 0) {
			$("#editor-placeholder").css({
				"left": left + "px"
			});
		} else {
			$("#editor-placeholder").animate({
				"left": left + "px"
			}, speed);
		}
	}

	function isFullScreen() {
		return (m.localStorage["VIEW_TYPEWRITER_SCROLLING_PLUS"] == 1 && m.win.isFullscreen && m.localStorage.VIEWPORT_SOURCES_PANE_VISIBLE == 0 && m.localStorage.VIEWPORT_DOCUMENT_PANE_VISIBLE == 0 && m.localStorage.VIEWPORT_MODULE_PANE_VISIBLE == 0)
	}

	return {
		_init: _init,
		toggleTSP: toggleTSP
	};
};